import time
import sqlite3
import shutil
import os
import sys

def main(dbname):
    dbms = sqlite3.connect(dbname);
    
    #lines below can be commented to set-up runs with and without indexes
    addIndexes(dbms)
    #dropIndexes(dbms);

    QA = """SELECT m.Title, COUNT(*)  as numActors FROM
	    Movies m inner join Casts c on  m.mid = c.mid
	    GROUP BY c.mid
	    HAVING COUNT(*) > 10"""
    QB = """SELECT a.FName, a.LName from
            Actors a inner join
            (SELECT Distinct(aid) as actorId
            FROM Casts c inner join Movies m on m.mid = c.mid
            where m.Title = 'Donnie Darko') j on j.actorId = a.aid"""
    QC = """SELECT a.FName, a.LName, mTitle FROM
            Actors a 
            inner join
            (
                SELECT a2.aid as actId, m2.title as mTitle FROM
                Actors a2 inner join Casts c on a2.aid = c.aid
                inner join Movies m2 on  m2.mid = c.mid
                GROUP BY a2.aid ORDER BY m2.Year DESC
            ) ON actId = a.aid
            """
    #Iterate over queries
    for query in [('QA',QA),('QB',QB),('QC',QC)]:
        queryStr = query[1];
        queryLabel = query[0];
        for i in range(0,5):
          start = time.clock()
          rowCounter = runSelect(queryStr, dbms);
          print "Rows returned "+queryLabel+": ",rowCounter
          print "Processing time "+queryLabel+": ", end
          end = time.clock() - start
    dbms.close();
    
def runSelect(queryString, dbms):
    cursor = dbms.cursor()
    cursor.execute(queryString)
    returned = cursor.fetchall()
    cursor.close()
    return len(returned)
    

def addIndexes(dbms):
    #re-add indexes which are to be tested.
    cursor = dbms.cursor()
    cursor.execute("CREATE INDEX actorId_movieId on Casts(aid,mid)")
    cursor.execute("CREATE INDEX Act_id on Actors(aid)")
    cursor.execute("CREATE INDEX Mov_Id on Movies(mid)")
#This is the index which was added for part (E)
    cursor.execute("CREATE INDEX Casts_MovieId on Casts(mid)")

def dropIndexes(dbms):
    #drop indexes originally in the database.
    cursor = dbms.cursor()
    cursor.execute("DROP INDEX actorId_movieId")
    cursor.execute("DROP INDEX Act_id")
    cursor.execute("DROP INDEX Mov_Id")

if __name__ == '__main__':
    main(sys.argv[1]);
